@@ -25,37 +25,43 @@
 
 /******************************************************************************/
 
-(( ) => {
+(async ( ) => {
     const params = new URL(document.location).searchParams;
     const assetKey = params.get('url');
     if ( assetKey === null ) { return; }
 
-    vAPI.messaging.send(
-        'default',
-        {
-            what : 'getAssetContent',
-            url: assetKey,
-        },
-        details => {
-            cmEditor.setValue(details && details.content || '');
-            if ( details.sourceURL ) {
-                const a = document.querySelector('.cm-search-widget .sourceURL');
-                a.setAttribute('href', details.sourceURL);
-                a.setAttribute('title', details.sourceURL);
-            }
-        }
-    );
-
-    const cmEditor = new CodeMirror(
-        document.getElementById('content'),
-        {
-            autofocus: true,
-            lineNumbers: true,
-            lineWrapping: true,
-            readOnly: true,
-            styleActiveLine: true,
-        }
-    );
+    const cmEditor = new CodeMirror(document.getElementById('content'), {
+        autofocus: true,
+        foldGutter: true,
+        gutters: [ 'CodeMirror-linenumbers', 'CodeMirror-foldgutter' ],
+        lineNumbers: true,
+        lineWrapping: true,
+        matchBrackets: true,
+        maxScanLines: 1,
+        readOnly: true,
+        styleActiveLine: true,
+    });
 
     uBlockDashboard.patchCodeMirrorEditor(cmEditor);
+
+    const hints = await vAPI.messaging.send('dashboard', {
+        what: 'getAutoCompleteDetails'
+    });
+    if ( hints instanceof Object ) {
+        const mode = cmEditor.getMode();
+        if ( mode.setHints instanceof Function ) {
+            mode.setHints(hints);
+        }
+    }
+
+    const details = await vAPI.messaging.send('default', {
+        what : 'getAssetContent',
+        url: assetKey,
+    });
+    cmEditor.setValue(details && details.content || '');
+    if ( details.sourceURL ) {
+        const a = document.querySelector('.cm-search-widget .sourceURL');
+        a.setAttribute('href', details.sourceURL);
+        a.setAttribute('title', details.sourceURL);
+    }
 })();
