@@ -33,43 +33,54 @@ if ( vAPI.webextFlavor === undefined ) {
 
 /******************************************************************************/
 
-const µBlock = (function() { // jshint ignore:line
+const µBlock = (( ) => { // jshint ignore:line
 
     const hiddenSettingsDefault = {
+        allowGenericProceduralFilters: false,
         assetFetchTimeout: 30,
         autoCommentFilterTemplate: '{{date}} {{origin}}',
         autoUpdateAssetFetchPeriod: 120,
+        autoUpdateDelayAfterLaunch: 180,
         autoUpdatePeriod: 7,
+        benchmarkDatasetURL: 'unset',
+        blockingProfiles: '11111/#F00 11011/#C0F 11001/#00F 00001',
         cacheStorageAPI: 'unset',
         cacheStorageCompression: true,
         cacheControlForFirefox1376932: 'no-cache, no-store, must-revalidate',
+        cnameIgnoreList: 'unset',
+        cnameIgnore1stParty: true,
+        cnameIgnoreExceptions: true,
+        cnameIgnoreRootDocument: true,
+        cnameMaxTTL: 120,
+        cnameReplayFullURL: false,
+        cnameUncloak: true,
+        cnameUncloakProxied: false,
         consoleLogLevel: 'unset',
         debugScriptlets: false,
+        debugScriptletInjector: false,
         disableWebAssembly: false,
+        extensionUpdateForceReload: false,
         ignoreRedirectFilters: false,
         ignoreScriptInjectFilters: false,
+        filterAuthorMode: false,
+        loggerPopupType: 'popup',
         manualUpdateAssetFetchPeriod: 500,
         popupFontSize: 'unset',
+        popupPanelDisabledSections: 0,
+        popupPanelLockedSections: 0,
+        popupPanelHeightMode: 0,
         requestJournalProcessPeriod: 1000,
-        selfieAfter: 11,
+        selfieAfter: 3,
         strictBlockingBypassDuration: 120,
         suspendTabsUntilReady: 'unset',
-        userResourcesLocation: 'unset'
+        uiPopupConfig: 'undocumented',
+        uiFlavor: 'unset',
+        uiStyles: 'unset',
+        updateAssetBypassBrowserCache: false,
+        userResourcesLocation: 'unset',
     };
 
-    const whitelistDefault = [
-        'about-scheme',
-        'chrome-extension-scheme',
-        'chrome-scheme',
-        'moz-extension-scheme',
-        'opera-scheme',
-        'vivaldi-scheme',
-        'wyciwyg-scheme',   // Firefox's "What-You-Cache-Is-What-You-Get"
-    ];
-
     return {
-        firstInstall: false,
-
         userSettings: {
             advancedUserEnabled: false,
             alwaysDetachLogger: true,
@@ -82,39 +93,19 @@ const µBlock = (function() { // jshint ignore:line
             externalLists: [],
             firewallPaneMinimized: true,
             hyperlinkAuditingDisabled: true,
-            ignoreGenericCosmeticFilters: false,
+            ignoreGenericCosmeticFilters: vAPI.webextFlavor.soup.has('mobile'),
             largeMediaSize: 50,
             parseAllABPHideFilters: true,
+            popupPanelSections: 0b111,
             prefetchingDisabled: true,
             requestLogMaxEntries: 1000,
             showIconBadge: true,
             tooltipsDisabled: false,
-            webrtcIPAddressHidden: false
+            webrtcIPAddressHidden: false,
         },
 
         hiddenSettingsDefault: hiddenSettingsDefault,
-        hiddenSettings: (function() {
-            const out = Object.assign({}, hiddenSettingsDefault);
-            const json = vAPI.localStorage.getItem('immediateHiddenSettings');
-            if ( typeof json !== 'string' ) { return out; }
-            try {
-                const o = JSON.parse(json);
-                if ( o instanceof Object ) {
-                    for ( const k in o ) {
-                        if ( out.hasOwnProperty(k) ) { out[k] = o[k]; }
-                    }
-                    self.log.verbosity = out.consoleLogLevel;
-                    if ( typeof out.suspendTabsUntilReady === 'boolean' ) {
-                        out.suspendTabsUntilReady = out.suspendTabsUntilReady
-                            ? 'yes'
-                            : 'unset';
-                    }
-                }
-            }
-            catch(ex) {
-            }
-            return out;
-        })(),
+        hiddenSettings: Object.assign({}, hiddenSettingsDefault),
 
         // Features detection.
         privacySettingsSupported: vAPI.browserSettings instanceof Object,
@@ -124,28 +115,47 @@ const µBlock = (function() { // jshint ignore:line
 
         // https://github.com/chrisaljoudi/uBlock/issues/180
         // Whitelist directives need to be loaded once the PSL is available
-        netWhitelist: {},
+        netWhitelist: new Map(),
         netWhitelistModifyTime: 0,
-        netWhitelistDefault: whitelistDefault.join('\n'),
+        netWhitelistDefault: [
+            'about-scheme',
+            'chrome-extension-scheme',
+            'chrome-scheme',
+            'edge-scheme',
+            'moz-extension-scheme',
+            'opera-scheme',
+            'vivaldi-scheme',
+            'wyciwyg-scheme',   // Firefox's "What-You-Cache-Is-What-You-Get"
+        ],
 
         localSettings: {
             blockedRequestCount: 0,
-            allowedRequestCount: 0
+            allowedRequestCount: 0,
         },
         localSettingsLastModified: 0,
         localSettingsLastSaved: 0,
 
         // Read-only
         systemSettings: {
-            compiledMagic: 14,  // Increase when compiled format changes
-            selfieMagic: 14     // Increase when selfie format changes
+            compiledMagic: 28,  // Increase when compiled format changes
+            selfieMagic: 28,    // Increase when selfie format changes
         },
 
+        // https://github.com/uBlockOrigin/uBlock-issues/issues/759#issuecomment-546654501
+        //   The assumption is that cache storage state reflects whether
+        //   compiled or selfie assets are available or not. The properties
+        //   below is to no longer rely on this assumption -- though it's still
+        //   not clear how the assumption could be wrong, and it's still not
+        //   clear whether relying on those properties will really solve the
+        //   issue. It's just an attempt at hardening.
+        compiledFormatChanged: false,
+        selfieIsInvalid: false,
+
         restoreBackupSettings: {
             lastRestoreFile: '',
             lastRestoreTime: 0,
             lastBackupFile: '',
-            lastBackupTime: 0
+            lastBackupTime: 0,
         },
 
         commandShortcuts: new Map(),
@@ -153,7 +163,7 @@ const µBlock = (function() { // jshint ignore:line
         // Allows to fully customize uBO's assets, typically set through admin
         // settings. The content of 'assets.json' will also tell which filter
         // lists to enable by default when uBO is first installed.
-        assetsBootstrapLocation: 'assets/assets.json',
+        assetsBootstrapLocation: undefined,
 
         userFiltersPath: 'user-filters',
         pslAssetKey: 'public_suffix_list.dat',
@@ -161,6 +171,10 @@ const µBlock = (function() { // jshint ignore:line
         selectedFilterLists: [],
         availableFilterLists: {},
 
+        // https://github.com/uBlockOrigin/uBlock-issues/issues/974
+        //   This can be used to defer filtering decision-making.
+        readyToFilter: false,
+
         pageStores: new Map(),
         pageStoresToken: 0,
 
@@ -171,18 +185,26 @@ const µBlock = (function() { // jshint ignore:line
 
         apiErrorCount: 0,
 
-        mouseEventRegister: {
-            tabId: '',
-            x: -1,
-            y: -1,
-            url: ''
+        maybeGoodPopup: {
+            tabId: 0,
+            url: '',
         },
 
-        epickerTarget: '',
-        epickerZap: false,
-        epickerEprom: null,
+        epickerArgs: {
+            eprom: null,
+            mouse: false,
+            target: '',
+            zap: false,
+        },
 
         scriptlets: {},
+
+        cspNoInlineScript: "script-src 'unsafe-eval' * blob: data:",
+        cspNoScripting: 'script-src http: https:',
+        cspNoInlineFont: 'font-src *',
+
+        liveBlockingProfiles: [],
+        blockingProfileColorCache: new Map(),
     };
 
 })();
