@@ -39,7 +39,7 @@
 // indexedDB.
 
 // https://bugzilla.mozilla.org/show_bug.cgi?id=1371255
-//   Firefox-specific: we use indexedDB because chrome.storage.local() has
+//   Firefox-specific: we use indexedDB because browser.storage.local() has
 //   poor performance in Firefox.
 // https://github.com/uBlockOrigin/uBlock-issues/issues/328
 //   Use IndexedDB for Chromium as well, to take advantage of LZ4
@@ -54,64 +54,15 @@
 
     const STORAGE_NAME = 'uBlock0CacheStorage';
 
-    // Default to webext storage. Wrapped into promises if the API does not
-    // support returning promises.
-    const promisified = (( ) => {
-        try {
-            return browser.storage.local.get('_') instanceof Promise;
-        }
-        catch(ex) {
-        }
-        return false;
-    })();
-
+    // Default to webext storage.
+    const localStorage = webext.storage.local;
     const api = {
         name: 'browser.storage.local',
-        get: promisified ?
-            browser.storage.local.get :
-            function(keys) {
-                return new Promise(resolve => {
-                    browser.storage.local.get(keys, bin => {
-                        resolve(bin);
-                    });
-                });
-            },
-        set: promisified ?
-            browser.storage.local.set :
-            function(keys) {
-                return new Promise(resolve => {
-                    browser.storage.local.set(keys, ( ) => {
-                        resolve();
-                    });
-                });
-            },
-        remove: promisified ?
-            browser.storage.local.remove :
-            function(keys) {
-                return new Promise(resolve => {
-                    browser.storage.local.remove(keys, ( ) => {
-                        resolve();
-                    });
-                });
-            },
-        clear: promisified ?
-            browser.storage.local.clear :
-            function() {
-                return new Promise(resolve => {
-                    browser.storage.local.clear(( ) => {
-                        resolve();
-                    });
-                });
-            },
-        getBytesInUse: promisified ?
-            browser.storage.local.getBytesInUse :
-            function(keys) {
-                return new Promise(resolve => {
-                    browser.storage.local.getBytesInUse(keys, count => {
-                        resolve(count);
-                    });
-                });
-            },
+        get: localStorage.get,
+        set: localStorage.set,
+        remove: localStorage.remove,
+        clear: localStorage.clear,
+        getBytesInUse: localStorage.getBytesInUse,
         select: function(selectedBackend) {
             let actualBackend = selectedBackend;
             if ( actualBackend === undefined || actualBackend === 'unset' ) {
@@ -139,7 +90,7 @@
     };
 
     // Reassign API entries to that of indexedDB-based ones
-    const selectIDB = function() {
+    const selectIDB = async function() {
         let db;
         let dbPromise;
         let dbTimer;
@@ -244,7 +195,7 @@
             return dbPromise;
         };
 
-        const getFromDb = function(keys, keyvalStore, callback) {
+        const getFromDb = async function(keys, keyvalStore, callback) {
             if ( typeof callback !== 'function' ) { return; }
             if ( keys.length === 0 ) { return callback(keyvalStore); }
             const promises = [];
@@ -261,7 +212,8 @@
                     })
                 );
             };
-            getDb().then(db => {
+            try {
+                const db = await getDb();
                 if ( !db ) { return callback(); }
                 const transaction = db.transaction(STORAGE_NAME, 'readonly');
                 transaction.oncomplete =
@@ -277,29 +229,29 @@
                     req.onsuccess = gotOne;
                     req.onerror = noopfn;
                 }
-            }).catch(reason => {
+            }
+            catch(reason) {
                 console.info(`cacheStorage.getFromDb() failed: ${reason}`);
                 callback();
-            });
+            }
         };
 
-        const visitAllFromDb = function(visitFn) {
-            getDb().then(db => {
-                if ( !db ) { return visitFn(); }
-                const transaction = db.transaction(STORAGE_NAME, 'readonly');
-                transaction.oncomplete =
-                transaction.onerror =
-                transaction.onabort = ( ) => visitFn();
-                const table = transaction.objectStore(STORAGE_NAME);
-                const req = table.openCursor();
-                req.onsuccess = function(ev) {
-                    let cursor = ev.target && ev.target.result;
-                    if ( !cursor ) { return; }
-                    let entry = cursor.value;
-                    visitFn(entry);
-                    cursor.continue();
-                };
-            });
+        const visitAllFromDb = async function(visitFn) {
+            const db = await getDb();
+            if ( !db ) { return visitFn(); }
+            const transaction = db.transaction(STORAGE_NAME, 'readonly');
+            transaction.oncomplete =
+            transaction.onerror =
+            transaction.onabort = ( ) => visitFn();
+            const table = transaction.objectStore(STORAGE_NAME);
+            const req = table.openCursor();
+            req.onsuccess = function(ev) {
+                let cursor = ev.target && ev.target.result;
+                if ( !cursor ) { return; }
+                let entry = cursor.value;
+                visitFn(entry);
+                cursor.continue();
+            };
         };
 
         const getAllFromDb = function(callback) {
@@ -335,7 +287,7 @@
         //   https://developer.mozilla.org/en-US/docs/Web/API/IDBDatabase/transaction
         //   https://developer.mozilla.org/en-US/docs/Web/API/IDBObjectStore/put
 
-        const putToDb = function(keyvalStore, callback) {
+        const putToDb = async function(keyvalStore, callback) {
             if ( typeof callback !== 'function' ) {
                 callback = noopfn;
             }
@@ -359,67 +311,67 @@
                     µBlock.lz4Codec.encode(key, data).then(handleEncodingResult)
                 );
             }
-            Promise.all(promises).then(results => {
+            const finish = ( ) => {
+                if ( callback === undefined ) { return; }
+                let cb = callback;
+                callback = undefined;
+                cb();
+            };
+            try {
+                const results = await Promise.all(promises);
                 const db = results[0];
                 if ( !db ) { return callback(); }
-                const finish = ( ) => {
-                    if ( callback === undefined ) { return; }
-                    let cb = callback;
-                    callback = undefined;
-                    cb();
-                };
-                try {
-                    const transaction = db.transaction(
-                        STORAGE_NAME,
-                        'readwrite'
-                    );
-                    transaction.oncomplete =
-                    transaction.onerror =
-                    transaction.onabort = finish;
-                    const table = transaction.objectStore(STORAGE_NAME);
-                    for ( const entry of entries ) {
-                        table.put(entry);
-                    }
-                } catch (ex) {
-                    finish();
+                const transaction = db.transaction(
+                    STORAGE_NAME,
+                    'readwrite'
+                );
+                transaction.oncomplete =
+                transaction.onerror =
+                transaction.onabort = finish;
+                const table = transaction.objectStore(STORAGE_NAME);
+                for ( const entry of entries ) {
+                    table.put(entry);
                 }
-            });
+            } catch (ex) {
+                finish();
+            }
         };
 
-        const deleteFromDb = function(input, callback) {
+        const deleteFromDb = async function(input, callback) {
             if ( typeof callback !== 'function' ) {
                 callback = noopfn;
             }
             const keys = Array.isArray(input) ? input.slice() : [ input ];
             if ( keys.length === 0 ) { return callback(); }
-            getDb().then(db => {
+            const finish = ( ) => {
+                if ( callback === undefined ) { return; }
+                let cb = callback;
+                callback = undefined;
+                cb();
+            };
+            try {
+                const db = await getDb();
                 if ( !db ) { return callback(); }
-                const finish = ( ) => {
-                    if ( callback === undefined ) { return; }
-                    let cb = callback;
-                    callback = undefined;
-                    cb();
-                };
-                try {
-                    const transaction = db.transaction(STORAGE_NAME, 'readwrite');
-                    transaction.oncomplete =
-                    transaction.onerror =
-                    transaction.onabort = finish;
-                    const table = transaction.objectStore(STORAGE_NAME);
-                    for ( const key of keys ) {
-                        table.delete(key);
-                    }
-                } catch (ex) {
-                    finish();
+                const transaction = db.transaction(STORAGE_NAME, 'readwrite');
+                transaction.oncomplete =
+                transaction.onerror =
+                transaction.onabort = finish;
+                const table = transaction.objectStore(STORAGE_NAME);
+                for ( const key of keys ) {
+                    table.delete(key);
                 }
-            });
+            } catch (ex) {
+                finish();
+            }
         };
 
-        const clearDb = function(callback) {
+        const clearDb = async function(callback) {
             if ( typeof callback !== 'function' ) {
                 callback = noopfn;
             }
-            getDb().then(db => {
+            try {
+                const db = await getDb();
+                if ( !db ) { return callback(); }
                 const transaction = db.transaction(STORAGE_NAME, 'readwrite');
                 transaction.oncomplete =
                 transaction.onerror =
@@ -427,77 +379,77 @@
                     callback();
                 };
                 transaction.objectStore(STORAGE_NAME).clear();
-            }).catch(reason => {
+            }
+            catch(reason) {
                 console.info(`cacheStorage.clearDb() failed: ${reason}`);
                 callback();
-            });
+            }
         };
 
-        return getDb().then(db => {
-            if ( !db ) { return false; }
-            api.name = 'indexedDB';
-            api.get = function get(keys) {
-                return new Promise(resolve => {
-                    if ( keys === null ) {
-                        return getAllFromDb(bin => resolve(bin));
-                    }
-                    let toRead, output = {};
-                    if ( typeof keys === 'string' ) {
-                        toRead = [ keys ];
-                    } else if ( Array.isArray(keys) ) {
-                        toRead = keys;
-                    } else /* if ( typeof keys === 'object' ) */ {
-                        toRead = Object.keys(keys);
-                        output = keys;
-                    }
-                    getFromDb(toRead, output, bin => resolve(bin));
-                });
-            };
-            api.set = function set(keys) {
-                return new Promise(resolve => {
-                    putToDb(keys, details => resolve(details));
-                });
-            };
-            api.remove = function remove(keys) {
-                return new Promise(resolve => {
-                    deleteFromDb(keys, ( ) => resolve());
-                });
-            };
-            api.clear = function clear() {
-                return new Promise(resolve => {
-                    clearDb(( ) => resolve());
-                });
-            };
-            api.getBytesInUse = function getBytesInUse() {
-                return Promise.resolve(0);
-            };
-            return true;
-        });
+        await getDb();
+        if ( !db ) { return false; }
+
+        api.name = 'indexedDB';
+        api.get = function get(keys) {
+            return new Promise(resolve => {
+                if ( keys === null ) {
+                    return getAllFromDb(bin => resolve(bin));
+                }
+                let toRead, output = {};
+                if ( typeof keys === 'string' ) {
+                    toRead = [ keys ];
+                } else if ( Array.isArray(keys) ) {
+                    toRead = keys;
+                } else /* if ( typeof keys === 'object' ) */ {
+                    toRead = Object.keys(keys);
+                    output = keys;
+                }
+                getFromDb(toRead, output, bin => resolve(bin));
+            });
+        };
+        api.set = function set(keys) {
+            return new Promise(resolve => {
+                putToDb(keys, details => resolve(details));
+            });
+        };
+        api.remove = function remove(keys) {
+            return new Promise(resolve => {
+                deleteFromDb(keys, ( ) => resolve());
+            });
+        };
+        api.clear = function clear() {
+            return new Promise(resolve => {
+                clearDb(( ) => resolve());
+            });
+        };
+        api.getBytesInUse = function getBytesInUse() {
+            return Promise.resolve(0);
+        };
+        return true;
     };
 
     // https://github.com/uBlockOrigin/uBlock-issues/issues/328
     //   Delete cache-related entries from webext storage.
-    const clearWebext = function() {
-        browser.storage.local.get('assetCacheRegistry', bin => {
-            if (
-                bin instanceof Object === false ||
-                bin.assetCacheRegistry instanceof Object === false
-            ) {
-                return;
+    const clearWebext = async function() {
+        const bin = await webext.storage.local.get('assetCacheRegistry');
+        if (
+            bin instanceof Object === false ||
+            bin.assetCacheRegistry instanceof Object === false
+        ) {
+            return;
+        }
+        const toRemove = [
+            'assetCacheRegistry',
+            'assetSourceRegistry',
+            'resourcesSelfie',
+            'selfie'
+        ];
+        for ( const key in bin.assetCacheRegistry ) {
+            if ( bin.assetCacheRegistry.hasOwnProperty(key) ) {
+                toRemove.push('cache/' + key);
             }
-            const toRemove = [
-                'assetCacheRegistry',
-                'assetSourceRegistry',
-                'resourcesSelfie',
-                'selfie'
-            ];
-            for ( const key in bin.assetCacheRegistry ) {
-                if ( bin.assetCacheRegistry.hasOwnProperty(key) ) {
-                    toRemove.push('cache/' + key);
-                }
-            }
-            browser.storage.local.remove(toRemove);
-        });
+        }
+        webext.storage.local.remove(toRemove);
     };
 
     const clearIDB = function() {
