@@ -19,13 +19,13 @@
     Home: https://github.com/gorhill/uBlock
 */
 
-/* global uDom */
+/* global uDom, faIconsInit */
+
+'use strict';
 
 /******************************************************************************/
 
-(function() {
-
-'use strict';
+(( ) => {
 
 /******************************************************************************/
 
@@ -39,31 +39,27 @@ self.cloud = {
 
 /******************************************************************************/
 
-var widget = uDom.nodeFromId('cloudWidget');
-if ( widget === null ) {
-    return;
-}
+const widget = uDom.nodeFromId('cloudWidget');
+if ( widget === null ) { return; }
 
 self.cloud.datakey = widget.getAttribute('data-cloud-entry') || '';
-if ( self.cloud.datakey === '' ) {
-    return;
-}
-
-var messaging = vAPI.messaging;
+if ( self.cloud.datakey === '' ) { return; }
 
 /******************************************************************************/
 
-var onCloudDataReceived = function(entry) {
-    if ( entry instanceof Object === false ) {
-        return;
-    }
+const fetchCloudData = async function() {
+    const entry = await vAPI.messaging.send('cloudWidget', {
+        what: 'cloudPull',
+        datakey: self.cloud.datakey,
+    });
+    if ( entry instanceof Object === false ) { return; }
 
     self.cloud.data = entry.data;
 
     uDom.nodeFromId('cloudPull').removeAttribute('disabled');
     uDom.nodeFromId('cloudPullAndMerge').removeAttribute('disabled');
 
-    var timeOptions = {
+    const timeOptions = {
         weekday: 'short',
         year: 'numeric',
         month: 'short',
@@ -74,7 +70,7 @@ var onCloudDataReceived = function(entry) {
         timeZoneName: 'short'
     };
 
-    var time = new Date(entry.tstamp);
+    const time = new Date(entry.tstamp);
     widget.querySelector('#cloudInfo').textContent =
         entry.source + '\n' +
         time.toLocaleString('fullwide', timeOptions);
@@ -82,40 +78,21 @@ var onCloudDataReceived = function(entry) {
 
 /******************************************************************************/
 
-var fetchCloudData = function() {
-    messaging.send(
-        'cloudWidget',
-        {
-            what: 'cloudPull',
-            datakey: self.cloud.datakey
-        },
-        onCloudDataReceived
-    );
-};
+const pushData = async function() {
+    if ( typeof self.cloud.onPush !== 'function' ) { return; }
 
-/******************************************************************************/
-
-var pushData = function() {
-    if ( typeof self.cloud.onPush !== 'function' ) {
-        return;
-    }
-    messaging.send(
-        'cloudWidget',
-        {
-            what: 'cloudPush',
-            datakey: self.cloud.datakey,
-            data: self.cloud.onPush()
-        },
-        function(error) {
-            var failed = typeof error === 'string';
-            document.getElementById('cloudPush')
-                    .classList
-                    .toggle('error', failed);
-            document.querySelector('#cloudError')
-                    .textContent = failed ? error : '';
-            fetchCloudData();
-        }
-    );
+    const error = await vAPI.messaging.send('cloudWidget', {
+        what: 'cloudPush',
+        datakey: self.cloud.datakey,
+        data: self.cloud.onPush(),
+    });
+    const failed = typeof error === 'string';
+    document.getElementById('cloudPush')
+            .classList
+            .toggle('error', failed);
+    document.querySelector('#cloudError')
+            .textContent = failed ? error : '';
+    fetchCloudData();
 };
 
 /******************************************************************************/
@@ -155,46 +132,34 @@ var closeOptions = function(ev) {
 
 /******************************************************************************/
 
-var submitOptions = function() {
-    var onOptions = function(options) {
-        if ( options instanceof Object === false ) {
-            return;
-        }
-        self.cloud.options = options;
-    };
-
-    messaging.send(
-        'cloudWidget',
-        {
-            what: 'cloudSetOptions',
-            options: {
-                deviceName: uDom.nodeFromId('cloudDeviceName').value
-            }
-        },
-        onOptions
-    );
+const submitOptions = async function() {
     uDom.nodeFromId('cloudOptions').classList.remove('show');
+
+    const options = await vAPI.messaging.send('cloudWidget', {
+        what: 'cloudSetOptions',
+        options: {
+            deviceName: uDom.nodeFromId('cloudDeviceName').value
+        },
+    });
+    if ( options instanceof Object ) {
+        self.cloud.options = options;
+    }
 };
 
 /******************************************************************************/
 
-var onInitialize = function(options) {
-    if ( typeof options !== 'object' || options === null ) {
-        return;
-    }
-
-    if ( !options.enabled ) {
-        return;
-    }
+const onInitialize = function(options) {
+    if ( options instanceof Object === false ) { return; }
+    if ( !options.enabled ) { return; }
     self.cloud.options = options;
 
-    var xhr = new XMLHttpRequest();
+    const xhr = new XMLHttpRequest();
     xhr.open('GET', 'cloud-ui.html', true);
     xhr.overrideMimeType('text/html;charset=utf-8');
     xhr.responseType = 'text';
     xhr.onload = function() {
         this.onload = null;
-        var parser = new DOMParser(),
+        const parser = new DOMParser(),
             parsed = parser.parseFromString(this.responseText, 'text/html'),
             fromParent = parsed.body;
         while ( fromParent.firstElementChild !== null ) {
@@ -203,15 +168,17 @@ var onInitialize = function(options) {
             );
         }
 
+        faIconsInit(widget);
+
         vAPI.i18n.render(widget);
         widget.classList.remove('hide');
 
-        uDom('#cloudPush').on('click', pushData);
+        uDom('#cloudPush').on('click', ( ) => { pushData(); });
         uDom('#cloudPull').on('click', pullData);
         uDom('#cloudPullAndMerge').on('click', pullAndMergeData);
         uDom('#cloudCog').on('click', openOptions);
         uDom('#cloudOptions').on('click', closeOptions);
-        uDom('#cloudOptionsSubmit').on('click', submitOptions);
+        uDom('#cloudOptionsSubmit').on('click', ( ) => { submitOptions(); });
         
         // Patch 2018-01-05: Must not assume this XHR will always be faster
         // than messaging
@@ -220,10 +187,12 @@ var onInitialize = function(options) {
     xhr.send();
 };
 
-messaging.send('cloudWidget', { what: 'cloudGetOptions' }, onInitialize);
+vAPI.messaging.send('cloudWidget', {
+    what: 'cloudGetOptions',
+}).then(options => {
+    onInitialize(options);
+});
 
 /******************************************************************************/
 
-// https://www.youtube.com/watch?v=aQFp67VoiDA
-
 })();
