@@ -23,7 +23,7 @@
 
 /******************************************************************************/
 
-µBlock.cosmeticFilteringEngine = (function(){
+µBlock.cosmeticFilteringEngine = (( ) => {
 
 /******************************************************************************/
 
@@ -191,7 +191,7 @@ const FilterContainer = function() {
     this.rePlainSelectorEscaped = /^[#.](?:\\[0-9A-Fa-f]+ |\\.|\w|-)+/;
     this.rePlainSelectorEx = /^[^#.\[(]+([#.][\w-]+)|([#.][\w-]+)$/;
     this.reEscapeSequence = /\\([0-9A-Fa-f]+ |.)/g;
-    this.reSimpleHighGeneric1 = /^[a-z]*\[[^[]+]$/;
+    this.reSimpleHighGeneric = /^(?:[a-z]*\[[^\]]+\]|\S+)$/;
     this.reHighMedium = /^\[href\^="https?:\/\/([^"]{8})[^"]*"\]$/;
 
     this.selectorCache = new Map();
@@ -204,6 +204,9 @@ const FilterContainer = function() {
     // specific filters
     this.specificFilters = new µb.staticExtFilteringEngine.HostnameBasedDB(2);
 
+    // temporary filters
+    this.sessionFilterDB = new µb.staticExtFilteringEngine.SessionDB();
+
     // low generic cosmetic filters, organized by id/class then simple/complex.
     this.lowlyGeneric = Object.create(null);
     this.lowlyGeneric.id = {
@@ -238,10 +241,12 @@ const FilterContainer = function() {
     // is to prevent repeated allocation/deallocation overheads -- the
     // constructors/destructors of javascript Set/Map is assumed to be costlier
     // than just calling clear() on these.
-    this.setRegister0 = new Set();
-    this.setRegister1 = new Set();
-    this.setRegister2 = new Set();
-    this.mapRegister0 = new Map();
+    this.$simpleSet = new Set();
+    this.$complexSet = new Set();
+    this.$specificSet = new Set();
+    this.$exceptionSet = new Set();
+    this.$proceduralSet = new Set();
+    this.$dummySet = new Set();
 
     this.reset();
 };
@@ -263,8 +268,8 @@ FilterContainer.prototype.reset = function() {
         this.selectorCacheTimer = null;
     }
 
-    // generic filters
-    this.hasGenericHide = false;
+    // whether there is at least one surveyor-based filter
+    this.needDOMSurveyor = false;
 
     // hostname, entity-based filters
     this.specificFilters.clear();
@@ -290,13 +295,11 @@ FilterContainer.prototype.freeze = function() {
     this.duplicateBuster.clear();
     this.specificFilters.collectGarbage();
 
-    this.hasGenericHide =
+    this.needDOMSurveyor =
         this.lowlyGeneric.id.simple.size !== 0 ||
         this.lowlyGeneric.id.complex.size !== 0 ||
         this.lowlyGeneric.cl.simple.size !== 0 ||
-        this.lowlyGeneric.cl.complex.size !== 0 ||
-        this.highlyGeneric.simple.dict.size !== 0 ||
-        this.highlyGeneric.complex.dict.size !== 0;
+        this.lowlyGeneric.cl.complex.size !== 0;
 
     this.highlyGeneric.simple.str = Array.from(this.highlyGeneric.simple.dict).join(',\n');
     this.highlyGeneric.simple.mru.reset();
@@ -322,8 +325,8 @@ FilterContainer.prototype.keyFromSelector = function(selector) {
     matches = this.rePlainSelectorEscaped.exec(selector);
     if ( matches === null ) { return; }
     key = '';
-    let escaped = matches[0],
-        beg = 0;
+    const escaped = matches[0];
+    let beg = 0;
     this.reEscapeSequence.lastIndex = 0;
     for (;;) {
         matches = this.reEscapeSequence.exec(escaped);
@@ -342,14 +345,12 @@ FilterContainer.prototype.keyFromSelector = function(selector) {
 
 /******************************************************************************/
 
-FilterContainer.prototype.compile = function(parsed, writer) {
+FilterContainer.prototype.compile = function(parser, writer) {
     // 1000 = cosmetic filtering
     writer.select(1000);
 
-    const hostnames = parsed.hostnames;
-    let i = hostnames.length;
-    if ( i === 0 ) {
-        this.compileGenericSelector(parsed, writer);
+    if ( parser.hasOptions() === false ) {
+        this.compileGenericSelector(parser, writer);
         return true;
     }
 
@@ -357,15 +358,15 @@ FilterContainer.prototype.compile = function(parsed, writer) {
     // Negated hostname means the filter applies to all non-negated hostnames
     // of same filter OR globally if there is no non-negated hostnames.
     let applyGlobally = true;
-    while ( i-- ) {
-        const hostname = hostnames[i];
-        if ( hostname.startsWith('~') === false ) {
+    for ( const { hn, not, bad } of parser.extOptions() ) {
+        if ( bad ) { continue; }
+        if ( not === false ) {
             applyGlobally = false;
         }
-        this.compileSpecificSelector(hostname, parsed, writer);
+        this.compileSpecificSelector(parser, hn, not, writer);
     }
     if ( applyGlobally ) {
-        this.compileGenericSelector(parsed, writer);
+        this.compileGenericSelector(parser, writer);
     }
 
     return true;
@@ -373,48 +374,52 @@ FilterContainer.prototype.compile = function(parsed, writer) {
 
 /******************************************************************************/
 
-FilterContainer.prototype.compileGenericSelector = function(parsed, writer) {
-    if ( parsed.exception === false ) {
-        this.compileGenericHideSelector(parsed, writer);
+FilterContainer.prototype.compileGenericSelector = function(parser, writer) {
+    if ( parser.isException() ) {
+        this.compileGenericUnhideSelector(parser, writer);
     } else {
-        this.compileGenericUnhideSelector(parsed, writer);
+        this.compileGenericHideSelector(parser, writer);
     }
 };
 
 /******************************************************************************/
 
 FilterContainer.prototype.compileGenericHideSelector = function(
-    parsed,
+    parser,
     writer
 ) {
-    const selector = parsed.suffix;
-    const type = selector.charCodeAt(0);
+    const { raw, compiled, pseudoclass } = parser.result;
+    if ( compiled === undefined ) {
+        const who = writer.properties.get('assetKey') || '?';
+        µb.logger.writeOne({
+            realm: 'message',
+            type: 'error',
+            text: `Invalid generic cosmetic filter in ${who}: ${raw}`
+        });
+    }
+
+    const type = compiled.charCodeAt(0);
     let key;
 
+    // Simple selector-based CSS rule: no need to test for whether the
+    // selector is valid, the regex took care of this. Most generic selector
+    // falls into that category:
+    // - ###ad-bigbox
+    // - ##.ads-bigbox
     if ( type === 0x23 /* '#' */ ) {
-        key = this.keyFromSelector(selector);
-        // Simple selector-based CSS rule: no need to test for whether the
-        // selector is valid, the regex took care of this. Most generic
-        // selector falls into that category.
-        // - ###ad-bigbox
-        if ( key === selector ) {
+        key = this.keyFromSelector(compiled);
+        if ( key === compiled ) {
             writer.push([ 0, key.slice(1) ]);
             return;
         }
     } else if ( type === 0x2E /* '.' */ ) {
-        key = this.keyFromSelector(selector);
-        // Simple selector-based CSS rule: no need to test for whether the
-        // selector is valid, the regex took care of this. Most generic
-        // selector falls into that category.
-        // - ##.ads-bigbox
-        if ( key === selector ) {
+        key = this.keyFromSelector(compiled);
+        if ( key === compiled ) {
             writer.push([ 2, key.slice(1) ]);
             return;
         }
     }
 
-    const compiled = µb.staticExtFilteringEngine.compileSelector(selector);
-
     // Invalid cosmetic filter, possible reasons:
     // - Bad syntax
     // - Procedural filters (can't be generic): the compiled version of
@@ -425,19 +430,15 @@ FilterContainer.prototype.compileGenericHideSelector = function(
     // https://github.com/uBlockOrigin/uBlock-issues/issues/131
     //   Support generic procedural filters as per advanced settings.
     //   TODO: prevent double compilation.
-    if (
-        compiled === undefined ||
-        compiled !== selector &&
-        µb.staticExtFilteringEngine.compileSelector.pseudoclass !== true
-    ) {
+    if ( compiled !== raw && pseudoclass === false ) {
         if ( µb.hiddenSettings.allowGenericProceduralFilters === true ) {
-            return this.compileSpecificSelector('', parsed, writer);
+            return this.compileSpecificSelector(parser, '', false, writer);
         }
         const who = writer.properties.get('assetKey') || '?';
         µb.logger.writeOne({
             realm: 'message',
             type: 'error',
-            text: `Invalid generic cosmetic filter in ${who}: ##${selector}`
+            text: `Invalid generic cosmetic filter in ${who}: ##${raw}`
         });
         return;
     }
@@ -449,7 +450,7 @@ FilterContainer.prototype.compileGenericHideSelector = function(
         writer.push([
             type === 0x23 /* '#' */ ? 1 : 3,
             key.slice(1),
-            selector
+            compiled
         ]);
         return;
     }
@@ -457,13 +458,13 @@ FilterContainer.prototype.compileGenericHideSelector = function(
     // https://github.com/gorhill/uBlock/issues/909
     //   Anything which contains a plain id/class selector can be classified
     //   as a low generic cosmetic filter.
-    const matches = this.rePlainSelectorEx.exec(selector);
+    const matches = this.rePlainSelectorEx.exec(compiled);
     if ( matches !== null ) {
         const key = matches[1] || matches[2];
         writer.push([
             key.charCodeAt(0) === 0x23 /* '#' */ ? 1 : 3,
             key.slice(1),
-            selector
+            compiled
         ]);
         return;
     }
@@ -473,32 +474,27 @@ FilterContainer.prototype.compileGenericHideSelector = function(
     // For efficiency purpose, we will distinguish between simple and complex
     // selectors.
 
-    if ( this.reSimpleHighGeneric1.test(selector) ) {
-        writer.push([ 4 /* simple */, selector ]);
-        return;
-    }
-
-    if ( selector.indexOf(' ') === -1 ) {
-        writer.push([ 4 /* simple */, selector ]);
+    if ( this.reSimpleHighGeneric.test(compiled) ) {
+        writer.push([ 4 /* simple */, compiled ]);
     } else {
-        writer.push([ 5 /* complex */, selector ]);
+        writer.push([ 5 /* complex */, compiled ]);
     }
 };
 
 /******************************************************************************/
 
 FilterContainer.prototype.compileGenericUnhideSelector = function(
-    parsed,
+    parser,
     writer
 ) {
     // Procedural cosmetic filters are acceptable as generic exception filters.
-    const compiled = µb.staticExtFilteringEngine.compileSelector(parsed.suffix);
+    const { raw, compiled } = parser.result;
     if ( compiled === undefined ) {
         const who = writer.properties.get('assetKey') || '?';
         µb.logger.writeOne({
             realm: 'message',
             type: 'error',
-            text: `Invalid cosmetic filter in ${who}: #@#${parsed.suffix}`
+            text: `Invalid cosmetic filter in ${who}: #@#${raw}`
         });
         return;
     }
@@ -509,41 +505,41 @@ FilterContainer.prototype.compileGenericUnhideSelector = function(
     //   hostnames). No distinction is made between declarative and
     //   procedural selectors, since they really exist only to cancel
     //   out other cosmetic filters.
-    writer.push([ 8, '', 0b01, compiled ]);
+    writer.push([ 8, '', 0b001, compiled ]);
 };
 
 /******************************************************************************/
 
 FilterContainer.prototype.compileSpecificSelector = function(
+    parser,
     hostname,
-    parsed,
+    not,
     writer
 ) {
-    // https://github.com/chrisaljoudi/uBlock/issues/145
-    let unhide = parsed.exception ? 1 : 0;
-    if ( hostname.startsWith('~') ) {
-        hostname = hostname.slice(1);
-        unhide ^= 1;
-    }
-
-    const compiled = µb.staticExtFilteringEngine.compileSelector(parsed.suffix);
+    const { raw, compiled, exception } = parser.result;
     if ( compiled === undefined ) {
         const who = writer.properties.get('assetKey') || '?';
         µb.logger.writeOne({
             realm: 'message',
             type: 'error',
-            text: `Invalid cosmetic filter in ${who}: ##${parsed.suffix}`
+            text: `Invalid cosmetic filter in ${who}: ##${raw}`
         });
         return;
     }
 
+    // https://github.com/chrisaljoudi/uBlock/issues/145
+    let unhide = exception ? 1 : 0;
+    if ( not ) { unhide ^= 1; }
 
     let kind = 0;
     if ( unhide === 1 ) {
-        kind |= 0b01;     // Exception
+        kind |= 0b001;     // Exception
     }
     if ( compiled.charCodeAt(0) === 0x7B /* '{' */ ) {
-        kind |= 0b10;     // Procedural
+        kind |= 0b010;     // Procedural
+    }
+    if ( hostname === '*' ) {
+        kind |= 0b100;     // Applies everywhere
     }
 
     writer.push([ 8, hostname, kind, compiled ]);
@@ -626,8 +622,21 @@ FilterContainer.prototype.fromCompiledContent = function(reader, options) {
 
         // hash,  example.com, .promoted-tweet
         // hash,  example.*, .promoted-tweet
+        //
+        // https://github.com/uBlockOrigin/uBlock-issues/issues/803
+        //   Handle specific filters meant to apply everywhere, i.e. selectors
+        //   not to be injected conditionally through the DOM surveyor.
+        //   hash,  *, .promoted-tweet
         case 8:
-            this.specificFilters.store(args[1], args[2], args[3]);
+            if ( args[2] === 0b100 ) {
+                if ( this.reSimpleHighGeneric.test(args[3]) )
+                    this.highlyGeneric.simple.dict.add(args[3]);
+                else {
+                    this.highlyGeneric.complex.dict.add(args[3]);
+                }
+                break;
+            }
+            this.specificFilters.store(args[1], args[2] & 0b011, args[3]);
             break;
 
         default:
@@ -655,11 +664,21 @@ FilterContainer.prototype.skipGenericCompiledContent = function(reader) {
 
         switch ( args[0] ) {
 
-        // hash,  example.com, .promoted-tweet
-        // hash,  example.*, .promoted-tweet
+        // https://github.com/uBlockOrigin/uBlock-issues/issues/803
+        //   Handle specific filters meant to apply everywhere, i.e. selectors
+        //   not to be injected conditionally through the DOM surveyor.
+        //   hash,  *, .promoted-tweet
         case 8:
             this.duplicateBuster.add(fingerprint);
-            this.specificFilters.store(args[1], args[2], args[3]);
+            if ( args[2] === 0b100 ) {
+                if ( this.reSimpleHighGeneric.test(args[3]) )
+                    this.highlyGeneric.simple.dict.add(args[3]);
+                else {
+                    this.highlyGeneric.complex.dict.add(args[3]);
+                }
+                break;
+            }
+            this.specificFilters.store(args[1], args[2] & 0b011, args[3]);
             break;
 
         default:
@@ -688,7 +707,6 @@ FilterContainer.prototype.toSelfie = function() {
         acceptedCount: this.acceptedCount,
         discardedCount: this.discardedCount,
         specificFilters: this.specificFilters.toSelfie(),
-        hasGenericHide: this.hasGenericHide,
         lowlyGenericSID: Array.from(this.lowlyGeneric.id.simple),
         lowlyGenericCID: Array.from(this.lowlyGeneric.id.complex),
         lowlyGenericSCL: Array.from(this.lowlyGeneric.cl.simple),
@@ -704,7 +722,6 @@ FilterContainer.prototype.fromSelfie = function(selfie) {
     this.acceptedCount = selfie.acceptedCount;
     this.discardedCount = selfie.discardedCount;
     this.specificFilters.fromSelfie(selfie.specificFilters);
-    this.hasGenericHide = selfie.hasGenericHide;
     this.lowlyGeneric.id.simple = new Set(selfie.lowlyGenericSID);
     this.lowlyGeneric.id.complex = new Map(selfie.lowlyGenericCID);
     this.lowlyGeneric.cl.simple = new Set(selfie.lowlyGenericSCL);
@@ -713,6 +730,11 @@ FilterContainer.prototype.fromSelfie = function(selfie) {
     this.highlyGeneric.simple.str = selfie.highSimpleGenericHideArray.join(',\n');
     this.highlyGeneric.complex.dict = new Set(selfie.highComplexGenericHideArray);
     this.highlyGeneric.complex.str = selfie.highComplexGenericHideArray.join(',\n');
+    this.needDOMSurveyor =
+        selfie.lowlyGenericSID.length !== 0 ||
+        selfie.lowlyGenericCID.length !== 0 ||
+        selfie.lowlyGenericSCL.length !== 0 ||
+        selfie.lowlyGenericCCL.length !== 0;
     this.frozen = true;
 };
 
@@ -817,8 +839,15 @@ FilterContainer.prototype.pruneSelectorCacheAsync = function() {
 /******************************************************************************/
 
 FilterContainer.prototype.randomAlphaToken = function() {
-    return String.fromCharCode(Date.now() % 26 + 97) +
-           Math.floor(Math.random() * 982451653 + 982451653).toString(36);
+    const now = Date.now();
+    return String.fromCharCode(now % 26 + 97) +
+           Math.floor((1 + Math.random()) * now).toString(36);
+};
+
+/******************************************************************************/
+
+FilterContainer.prototype.getSession = function() {
+    return this.sessionFilterDB;
 };
 
 /******************************************************************************/
@@ -829,11 +858,11 @@ FilterContainer.prototype.retrieveGenericSelectors = function(request) {
 
     //console.time('cosmeticFilteringEngine.retrieveGenericSelectors');
 
-    const simpleSelectors = this.setRegister0;
-    const complexSelectors = this.setRegister1;
+    const simpleSelectors = this.$simpleSet;
+    const complexSelectors = this.$complexSet;
 
     const cacheEntry = this.selectorCache.get(request.hostname);
-    const previousHits = cacheEntry && cacheEntry.cosmetic || this.setRegister2;
+    const previousHits = cacheEntry && cacheEntry.cosmetic || this.$dummySet;
 
     for ( const type in this.lowlyGeneric ) {
         const entry = this.lowlyGeneric[type];
@@ -890,6 +919,10 @@ FilterContainer.prototype.retrieveGenericSelectors = function(request) {
         excepted,
     };
 
+    // Important: always clear used registers before leaving.
+    simpleSelectors.clear();
+    complexSelectors.clear();
+
     // Cache and inject (if user stylesheets supported) looked-up low generic
     // cosmetic filters.
     if (
@@ -922,18 +955,15 @@ FilterContainer.prototype.retrieveGenericSelectors = function(request) {
             out.complex = [];
         }
         out.injected = injected.join(',\n');
-        vAPI.insertCSS(request.tabId, {
+        vAPI.tabs.insertCSS(request.tabId, {
             code: out.injected + '\n{display:none!important;}',
             cssOrigin: 'user',
             frameId: request.frameId,
-            runAt: 'document_start'
+            matchAboutBlank: true,
+            runAt: 'document_start',
         });
     }
 
-    // Important: always clear used registers before leaving.
-    this.setRegister0.clear();
-    this.setRegister1.clear();
-
     //console.timeEnd('cosmeticFilteringEngine.retrieveGenericSelectors');
 
     return out;
@@ -945,8 +975,6 @@ FilterContainer.prototype.retrieveSpecificSelectors = function(
     request,
     options
 ) {
-    //console.time('cosmeticFilteringEngine.retrieveSpecificSelectors');
-
     const hostname = request.hostname;
     const cacheEntry = this.selectorCache.get(hostname);
 
@@ -970,12 +998,16 @@ FilterContainer.prototype.retrieveSpecificSelectors = function(
         highGenericHideComplex: '',
         injectedHideFilters: '',
         networkFilters: '',
-        noDOMSurveying: this.hasGenericHide === false,
+        noDOMSurveying: this.needDOMSurveyor === false,
         proceduralFilters: []
     };
 
     if ( options.noCosmeticFiltering !== true ) {
-        const specificSet = this.setRegister1;
+        const specificSet = this.$specificSet;
+        const proceduralSet = this.$proceduralSet;
+        const exceptionSet = this.$exceptionSet;
+        const dummySet = this.$dummySet;
+
         // Cached cosmetic filters: these are always declarative.
         if ( cacheEntry !== undefined ) {
             cacheEntry.retrieve('cosmetic', specificSet);
@@ -985,17 +1017,35 @@ FilterContainer.prototype.retrieveSpecificSelectors = function(
             }
         }
 
-        const exceptionSet = this.setRegister0;
-        const proceduralSet = this.setRegister2;
+        // Retrieve temporary filters
+        if ( this.sessionFilterDB.isNotEmpty ) {
+            this.sessionFilterDB.retrieve([ null, exceptionSet ]);
+        }
 
+        // Retrieve filters with a non-empty hostname
         this.specificFilters.retrieve(
             hostname,
-            [ specificSet, exceptionSet, proceduralSet, exceptionSet ]
+            options.noSpecificCosmeticFiltering !== true
+                ? [ specificSet, exceptionSet, proceduralSet, exceptionSet ]
+                : [ dummySet, exceptionSet ],
+            1
         );
+        // Retrieve filters with an empty hostname
+        this.specificFilters.retrieve(
+            hostname,
+            options.noGenericCosmeticFiltering !== true
+                ? [ specificSet, exceptionSet, proceduralSet, exceptionSet ]
+                : [ dummySet, exceptionSet ],
+            2
+        );
+        // Retrieve filters with a non-empty entity
         if ( request.entity !== '' ) {
             this.specificFilters.retrieve(
                 `${hostname.slice(0, -request.domain.length)}${request.entity}`,
-                [ specificSet, exceptionSet, proceduralSet, exceptionSet ]
+                options.noSpecificCosmeticFiltering !== true
+                    ? [ specificSet, exceptionSet, proceduralSet, exceptionSet ]
+                    : [ dummySet, exceptionSet ],
+                1
             );
         }
 
@@ -1059,9 +1109,10 @@ FilterContainer.prototype.retrieveSpecificSelectors = function(
         }
 
         // Important: always clear used registers before leaving.
-        this.setRegister0.clear();
-        this.setRegister1.clear();
-        this.setRegister2.clear();
+        specificSet.clear();
+        proceduralSet.clear();
+        exceptionSet.clear();
+        dummySet.clear();
     }
 
     // CSS selectors for collapsible blocked elements
@@ -1101,21 +1152,20 @@ FilterContainer.prototype.retrieveSpecificSelectors = function(
             code: '',
             cssOrigin: 'user',
             frameId: request.frameId,
-            runAt: 'document_start'
+            matchAboutBlank: true,
+            runAt: 'document_start',
         };
         if ( out.injectedHideFilters.length !== 0 ) {
             details.code = out.injectedHideFilters + '\n{display:none!important;}';
-            vAPI.insertCSS(request.tabId, details);
+            vAPI.tabs.insertCSS(request.tabId, details);
         }
         if ( out.networkFilters.length !== 0 ) {
             details.code = out.networkFilters + '\n{display:none!important;}';
-            vAPI.insertCSS(request.tabId, details);
+            vAPI.tabs.insertCSS(request.tabId, details);
             out.networkFilters = '';
         }
     }
 
-    //console.timeEnd('cosmeticFilteringEngine.retrieveSpecificSelectors');
-
     return out;
 };
 
