@@ -29,8 +29,6 @@
 
 /******************************************************************************/
 
-const messaging = vAPI.messaging;
-
 const mergeView = new CodeMirror.MergeView(
     document.querySelector('.codeMirrorMergeContainer'),
     {
@@ -105,7 +103,7 @@ let differ;
 
 const updateOverlay = (function() {
     let reFilter;
-    let mode = {
+    const mode = {
         token: function(stream) {
             if ( reFilter !== undefined ) {
                 reFilter.lastIndex = stream.pos;
@@ -190,8 +188,8 @@ const rulesToDoc = function(clearHistory) {
 /******************************************************************************/
 
 const filterRules = function(key) {
+    const filter = uDom.nodeFromSelector('#ruleFilter input').value;
     let rules = unfilteredRules[key].rules;
-    let filter = uDom.nodeFromSelector('#ruleFilter input').value;
     if ( filter !== '' ) {
         rules = rules.slice();
         let i = rules.length;
@@ -235,17 +233,14 @@ const renderRules = (( ) => {
 
 /******************************************************************************/
 
-const applyDiff = function(permanent, toAdd, toRemove) {
-    messaging.send(
-        'dashboard',
-        {
-            what: 'modifyRuleset',
-            permanent: permanent,
-            toAdd: toAdd,
-            toRemove: toRemove
-        },
-        renderRules
-    );
+const applyDiff = async function(permanent, toAdd, toRemove) {
+    const details = await vAPI.messaging.send('dashboard', {
+        what: 'modifyRuleset',
+        permanent: permanent,
+        toAdd: toAdd,
+        toRemove: toRemove,
+    });
+    renderRules(details);
 };
 
 /******************************************************************************/
@@ -266,13 +261,13 @@ mergeView.options.revertChunk = function(
     }
     if ( typeof fromStart.ch !== 'number' ) { fromStart.ch = 0; }
     if ( fromEnd.ch !== 0 ) { fromEnd.line += 1; }
-    let toAdd = from.getRange(
+    const toAdd = from.getRange(
         { line: fromStart.line, ch: 0 },
         { line: fromEnd.line, ch: 0 }
     );
     if ( typeof toStart.ch !== 'number' ) { toStart.ch = 0; }
     if ( toEnd.ch !== 0 ) { toEnd.line += 1; }
-    let toRemove = to.getRange(
+    const toRemove = to.getRange(
         { line: toStart.line, ch: 0 },
         { line: toEnd.line, ch: 0 }
     );
@@ -282,7 +277,7 @@ mergeView.options.revertChunk = function(
 /******************************************************************************/
 
 function handleImportFilePicker() {
-    let fileReaderOnLoadHandler = function() {
+    const fileReaderOnLoadHandler = function() {
         if ( typeof this.result !== 'string' || this.result === '' ) { return; }
         // https://github.com/chrisaljoudi/uBlock/issues/757
         // Support RequestPolicy rule syntax
@@ -295,10 +290,10 @@ function handleImportFilePicker() {
         }
         applyDiff(false, result, '');
     };
-    let file = this.files[0];
+    const file = this.files[0];
     if ( file === undefined || file.name === '' ) { return; }
     if ( file.type.indexOf('text') !== 0 ) { return; }
-    let fr = new FileReader();
+    const fr = new FileReader();
     fr.onload = fileReaderOnLoadHandler;
     fr.readAsText(file);
 }
@@ -306,7 +301,7 @@ function handleImportFilePicker() {
 /******************************************************************************/
 
 const startImportFilePicker = function() {
-    let input = document.getElementById('importFilePicker');
+    const input = document.getElementById('importFilePicker');
     // Reset to empty string, this will ensure an change event is properly
     // triggered if the user pick a file, even if it is the same as the last
     // one picked.
@@ -317,7 +312,7 @@ const startImportFilePicker = function() {
 /******************************************************************************/
 
 function exportUserRulesToFile() {
-    let filename = vAPI.i18n('rulesDefaultFileName')
+    const filename = vAPI.i18n('rulesDefaultFileName')
         .replace('{{datetime}}', uBlockDashboard.dateNowToSensibleString())
         .replace(/ +/g, '_');
     vAPI.download({
@@ -339,7 +334,7 @@ const onFilterChanged = (function() {
     const process = function() {
         timer = undefined;
         if ( mergeView.editor().isClean(cleanEditToken) === false ) { return; }
-        let filter = uDom.nodeFromSelector('#ruleFilter input').value;
+        const filter = uDom.nodeFromSelector('#ruleFilter input').value;
         if ( filter === last ) { return; }
         last = filter;
         if ( overlay !== null ) {
@@ -378,7 +373,7 @@ const onTextChanged = (( ) => {
             cleanEditToken = mergeView.editor().changeGeneration();
             isClean = true;
         }
-        diff.classList.toggle('editing', isClean === false);
+        document.body.classList.toggle('editing', isClean === false);
         diff.classList.toggle('dirty', mergeView.leftChunks().length !== 0);
         document.getElementById('editSaveButton')
                 .classList.toggle('disabled', isClean);
@@ -401,15 +396,15 @@ const onTextChanged = (( ) => {
 /******************************************************************************/
 
 const revertAllHandler = function() {
-    let toAdd = [], toRemove = [];
-    let left = mergeView.leftOriginal(),
-        edit = mergeView.editor();
-    for ( let chunk of mergeView.leftChunks() ) {
-        let addedLines = left.getRange(
+    const toAdd = [], toRemove = [];
+    const left = mergeView.leftOriginal();
+    const edit = mergeView.editor();
+    for ( const chunk of mergeView.leftChunks() ) {
+        const addedLines = left.getRange(
             { line: chunk.origFrom, ch: 0 },
             { line: chunk.origTo, ch: 0 }
         );
-        let removedLines = edit.getRange(
+        const removedLines = edit.getRange(
             { line: chunk.editFrom, ch: 0 },
             { line: chunk.editTo, ch: 0 }
         );
@@ -422,15 +417,15 @@ const revertAllHandler = function() {
 /******************************************************************************/
 
 const commitAllHandler = function() {
-    let toAdd = [], toRemove = [];
-    let left = mergeView.leftOriginal(),
-        edit = mergeView.editor();
-    for ( let chunk of mergeView.leftChunks() ) {
-        let addedLines = edit.getRange(
+    const toAdd = [], toRemove = [];
+    const left = mergeView.leftOriginal();
+    const edit = mergeView.editor();
+    for ( const chunk of mergeView.leftChunks() ) {
+        const addedLines = edit.getRange(
             { line: chunk.editFrom, ch: 0 },
             { line: chunk.editTo, ch: 0 }
         );
-        let removedLines = left.getRange(
+        const removedLines = left.getRange(
             { line: chunk.origFrom, ch: 0 },
             { line: chunk.origTo, ch: 0 }
         );
@@ -443,16 +438,16 @@ const commitAllHandler = function() {
 /******************************************************************************/
 
 const editSaveHandler = function() {
-    let editor = mergeView.editor();
-    let editText = editor.getValue().trim();
+    const editor = mergeView.editor();
+    const editText = editor.getValue().trim();
     if ( editText === cleanEditText ) {
         onTextChanged(true);
         return;
     }
     if ( differ === undefined ) { differ = new diff_match_patch(); }
-    let toAdd = [], toRemove = [];
-    let diffs = differ.diff_main(cleanEditText, editText);
-    for ( let diff of diffs ) {
+    const toAdd = [], toRemove = [];
+    const diffs = differ.diff_main(cleanEditText, editText);
+    for ( const diff of diffs ) {
         if ( diff[0] === 1 ) {
             toAdd.push(diff[1]);
         } else if ( diff[0] === -1 ) {
@@ -485,7 +480,11 @@ self.hasUnsavedData = function() {
 
 /******************************************************************************/
 
-messaging.send('dashboard', { what: 'getRules' }, renderRules);
+vAPI.messaging.send('dashboard', {
+    what: 'getRules',
+}).then(details => {
+    renderRules(details);
+});
 
 // Handle user interaction
 uDom('#importButton').on('click', startImportFilePicker);
