@@ -24,12 +24,12 @@
 /******************************************************************************/
 
 (( ) => {
+// >>>>>>>> start of private namespace
 
 /******************************************************************************/
 
 if (
     typeof vAPI !== 'object' ||
-    vAPI.domFilterer instanceof Object === false ||
     vAPI.domWatcher instanceof Object === false
 ) {
     return;
@@ -45,6 +45,7 @@ let declarativeStyleStr;
 const proceduralDict = new Map();
 const exceptionDict = new Map();
 let exceptionStr;
+const proceduralExceptionDict = new Map();
 const nodesToProcess = new Set();
 const loggedSelectors = new Set();
 
@@ -129,13 +130,15 @@ const processDeclarativeStyle = function(out) {
         declarativeStyleStr = safeGroupSelectors(declarativeStyleDict.keys());
     }
     if ( document.querySelector(declarativeStyleStr) === null ) { return; }
-    for ( const [ selector, style ] of declarativeStyleDict ) {
+    for ( const selector of declarativeStyleDict.keys() ) {
         if ( safeQuerySelector(selector) === null ) { continue; }
-        const raw = `##${selector}:style(${style})`;
-        out.push(raw);
+        for ( const style of declarativeStyleDict.get(selector) ) {
+            const raw = `##${selector}:style(${style})`;
+            out.push(raw);
+            loggedSelectors.add(raw);
+        }
         declarativeStyleDict.delete(selector);
         declarativeStyleStr = undefined;
-        loggedSelectors.add(raw);
     }
 };
 
@@ -143,10 +146,10 @@ const processDeclarativeStyle = function(out) {
 
 const processProcedural = function(out) {
     if ( proceduralDict.size === 0 ) { return; }
-    for ( const entry of proceduralDict ) {
-        if ( entry[1].test() === false ) { continue; }
-        out.push(`##${entry[1].raw}`);
-        proceduralDict.delete(entry[0]);
+    for ( const [ raw, pselector ] of proceduralDict ) {
+        if ( pselector.hit === false ) { continue; }
+        out.push(`##${raw}`);
+        proceduralDict.delete(raw);
     }
 };
 
@@ -169,6 +172,17 @@ const processExceptions = function(out) {
 
 /******************************************************************************/
 
+const processProceduralExceptions = function(out) {
+    if ( proceduralExceptionDict.size === 0 ) { return; }
+    for ( const exception of proceduralExceptionDict.values() ) {
+        if ( exception.test() === false ) { continue; }
+        out.push(`#@#${exception.raw}`);
+        proceduralExceptionDict.delete(exception.raw);
+    }
+};
+
+/******************************************************************************/
+
 const processTimer = new vAPI.SafeAnimationFrame(( ) => {
     //console.time('dom logger/scanning for matches');
     processTimer.clear();
@@ -190,20 +204,20 @@ const processTimer = new vAPI.SafeAnimationFrame(( ) => {
     processDeclarativeStyle(toLog);
     processProcedural(toLog);
     processExceptions(toLog);
+    processProceduralExceptions(toLog);
 
     nodesToProcess.clear();
 
     if ( toLog.length === 0 ) { return; }
 
-    vAPI.messaging.send(
-        'scriptlets',
-        {
-            what: 'logCosmeticFilteringData',
-            frameURL: window.location.href,
-            frameHostname: window.location.hostname,
-            matchedSelectors: toLog,
-        }
-    );
+    const location = vAPI.effectiveSelf.location;
+
+    vAPI.messaging.send('scriptlets', {
+        what: 'logCosmeticFilteringData',
+        frameURL: location.href,
+        frameHostname: location.hostname,
+        matchedSelectors: toLog,
+    });
     //console.timeEnd('dom logger/scanning for matches');
 });
 
@@ -229,8 +243,13 @@ const handlers = {
         for ( const entry of (changes.declarative || []) ) {
             for ( let selector of entry[0].split(',\n') ) {
                 if ( entry[1] !== 'display:none!important;' ) {
-                    declarativeStyleDict.set(selector, entry[1]);
                     declarativeStyleStr = undefined;
+                    const styles = declarativeStyleDict.get(selector);
+                    if ( styles === undefined ) {
+                        declarativeStyleDict.set(selector, [ entry[1] ]);
+                        continue;
+                    }
+                    styles.push(entry[1]);
                     continue;
                 }
                 if ( loggedSelectors.has(selector) ) { continue; }
@@ -254,12 +273,19 @@ const handlers = {
         if ( Array.isArray(changes.exceptions) ) {
             for ( const selector of changes.exceptions ) {
                 if ( loggedSelectors.has(selector) ) { continue; }
-                if ( selector.charCodeAt(0) === 0x7B /* '{' */ ) {
-                    const details = JSON.parse(selector);
-                    exceptionDict.set(details.style[0], details.raw);
-                } else {
+                if ( selector.charCodeAt(0) !== 0x7B /* '{' */ ) {
                     exceptionDict.set(selector, selector);
+                    continue;
                 }
+                const details = JSON.parse(selector);
+                if ( details.action === 'style' ) {
+                    exceptionDict.set(details.selector, details.raw);
+                    continue;
+                }
+                proceduralExceptionDict.set(
+                    details.raw,
+                    vAPI.domFilterer.createProceduralFilter(details)
+                );
             }
             exceptionStr = undefined;
         }
@@ -270,6 +296,9 @@ const handlers = {
     },
 
     onDOMCreated: function() {
+        if ( vAPI.domFilterer instanceof Object === false ) {
+            return shutdown();
+        }
         handlers.onFiltersetChanged(vAPI.domFilterer.getAllSelectors());
         vAPI.domFilterer.addListener(handlers);
         attributeObserver.observe(document.body, {
@@ -292,21 +321,43 @@ const handlers = {
 
 /******************************************************************************/
 
-const onMessage = function(msg) {
-    if ( msg.what === 'loggerDisabled' ) {
-        processTimer.clear();
-        attributeObserver.disconnect();
+const shutdown = function() {
+    processTimer.clear();
+    attributeObserver.disconnect();
+    if ( typeof vAPI !== 'object' ) { return; }
+    if ( vAPI.domFilterer instanceof Object ) {
         vAPI.domFilterer.removeListener(handlers);
+    }
+    if ( vAPI.domWatcher instanceof Object ) {
         vAPI.domWatcher.removeListener(handlers);
-        vAPI.messaging.removeChannelListener('domLogger', onMessage);
+    }
+    if ( vAPI.broadcastListener instanceof Object ) {
+        vAPI.broadcastListener.remove(broadcastListener);
     }
 };
-vAPI.messaging.addChannelListener('domLogger', onMessage);
+
+/******************************************************************************/
+
+const broadcastListener = msg => {
+    if ( msg.what === 'loggerDisabled' ) {
+        shutdown();
+    }
+};
+
+/******************************************************************************/
+
+vAPI.messaging.extend().then(extended => {
+    if ( extended !== true ) {
+        return shutdown();
+    }
+    vAPI.broadcastListener.add(broadcastListener);
+});
 
 vAPI.domWatcher.addListener(handlers);
 
 /******************************************************************************/
 
+// <<<<<<<< end of private namespace
 })();
 
 
