@@ -26,94 +26,6 @@
 /******************************************************************************/
 /******************************************************************************/
 
-/*! http://mths.be/cssescape v0.2.1 by @mathias | MIT license */
-;(function(root) {
-
-    if (!root.CSS) {
-        root.CSS = {};
-    }
-
-    var CSS = root.CSS;
-
-    var InvalidCharacterError = function(message) {
-        this.message = message;
-    };
-    InvalidCharacterError.prototype = new Error();
-    InvalidCharacterError.prototype.name = 'InvalidCharacterError';
-
-    if (!CSS.escape) {
-        // http://dev.w3.org/csswg/cssom/#serialize-an-identifier
-        CSS.escape = function(value) {
-            var string = String(value);
-            var length = string.length;
-            var index = -1;
-            var codeUnit;
-            var result = '';
-            var firstCodeUnit = string.charCodeAt(0);
-            while (++index < length) {
-                codeUnit = string.charCodeAt(index);
-                // Note: there’s no need to special-case astral symbols, surrogate
-                // pairs, or lone surrogates.
-
-                // If the character is NULL (U+0000), then throw an
-                // `InvalidCharacterError` exception and terminate these steps.
-                if (codeUnit === 0x0000) {
-                    throw new InvalidCharacterError(
-                        'Invalid character: the input contains U+0000.'
-                    );
-                }
-
-                if (
-                    // If the character is in the range [\1-\1F] (U+0001 to U+001F) or is
-                    // U+007F, […]
-                    (codeUnit >= 0x0001 && codeUnit <= 0x001F) || codeUnit === 0x007F ||
-                    // If the character is the first character and is in the range [0-9]
-                    // (U+0030 to U+0039), […]
-                    (index === 0 && codeUnit >= 0x0030 && codeUnit <= 0x0039) ||
-                    // If the character is the second character and is in the range [0-9]
-                    // (U+0030 to U+0039) and the first character is a `-` (U+002D), […]
-                    (
-                        index === 1 &&
-                        codeUnit >= 0x0030 && codeUnit <= 0x0039 &&
-                        firstCodeUnit === 0x002D
-                    )
-                ) {
-                    // http://dev.w3.org/csswg/cssom/#escape-a-character-as-code-point
-                    result += '\\' + codeUnit.toString(16) + ' ';
-                    continue;
-                }
-
-                // If the character is not handled by one of the above rules and is
-                // greater than or equal to U+0080, is `-` (U+002D) or `_` (U+005F), or
-                // is in one of the ranges [0-9] (U+0030 to U+0039), [A-Z] (U+0041 to
-                // U+005A), or [a-z] (U+0061 to U+007A), […]
-                if (
-                    codeUnit >= 0x0080 ||
-                    codeUnit === 0x002D ||
-                    codeUnit === 0x005F ||
-                    codeUnit >= 0x0030 && codeUnit <= 0x0039 ||
-                    codeUnit >= 0x0041 && codeUnit <= 0x005A ||
-                    codeUnit >= 0x0061 && codeUnit <= 0x007A
-                ) {
-                    // the character itself
-                    result += string.charAt(index);
-                    continue;
-                }
-
-                // Otherwise, the escaped character.
-                // http://dev.w3.org/csswg/cssom/#escape-a-character
-                result += '\\' + string.charAt(index);
-
-            }
-            return result;
-        };
-    }
-
-}(self));
-
-/******************************************************************************/
-/******************************************************************************/
-
 (( ) => {
 
 /******************************************************************************/
@@ -126,26 +38,26 @@ if (
     return;
 }
 
-var pickerRoot = document.getElementById(vAPI.sessionId);
+let pickerRoot = document.getElementById(vAPI.sessionId);
 if ( pickerRoot ) { return; }
 
-var pickerBody = null;
-var svgOcean = null;
-var svgIslands = null;
-var svgRoot = null;
-var dialog = null;
-var taCandidate = null;
+let pickerBody = null;
+let svgOcean = null;
+let svgIslands = null;
+let svgRoot = null;
+let dialog = null;
+let taCandidate = null;
 
-var netFilterCandidates = [];
-var cosmeticFilterCandidates = [];
+const netFilterCandidates = [];
+const cosmeticFilterCandidates = [];
 
-var targetElements = [];
-var candidateElements = [];
-var bestCandidateFilter = null;
+let targetElements = [];
+let candidateElements = [];
+let bestCandidateFilter = null;
 
-var lastNetFilterSession = window.location.host + window.location.pathname;
-var lastNetFilterHostname = '';
-var lastNetFilterUnion = '';
+const lastNetFilterSession = window.location.host + window.location.pathname;
+let lastNetFilterHostname = '';
+let lastNetFilterUnion = '';
 
 /******************************************************************************/
 
@@ -291,20 +203,28 @@ const mergeStrings = function(urls) {
             } else {
                 result.push(diff[1].replace(/\n+/g, ''));
             }
+            merged = result.join('');
         }
-        // Keep usage of wildcards to a sane level, too many of them can cause
-        // high overhead filters
-        merged =
-            result.join('')
-                .replace(/\*+$/, '')
-                .replace(/\*{2,}/g, '*')
-                .replace(/([^*]{1,2}\*)(?:[^*]{1,2}\*)+/g, '$1');
     }
+    // Keep usage of wildcards to a sane level, too many of them can cause
+    // high overhead filters
+    merged = merged.replace(/^\*+$/, '')
+                   .replace(/\*{2,}/g, '*')
+                   .replace(/([^*]{1,3}\*)(?:[^*]{1,3}\*)+/g, '$1');
     return merged;
 };
 
 /******************************************************************************/
 
+// Remove fragment part from a URL.
+
+const trimFragmentFromURL = function(url) {
+    const pos = url.indexOf('#');
+    return pos !== -1 ? url.slice(0, pos) : url;
+};
+
+/******************************************************************************/
+
 // https://github.com/gorhill/uBlock/issues/1897
 // Ignore `data:` URI, they can't be handled by an HTTP observer.
 
@@ -313,7 +233,9 @@ const backgroundImageURLFromElement = function(elem) {
     const bgImg = style.backgroundImage || '';
     const matches = /^url\((["']?)([^"']+)\1\)$/.exec(bgImg);
     const url = matches !== null && matches.length === 3 ? matches[2] : '';
-    return url.lastIndexOf('data:', 0) === -1 ? url.slice(0, 1024) : '';
+    return url.lastIndexOf('data:', 0) === -1
+        ? trimFragmentFromURL(url.slice(0, 1024))
+        : '';
 };
 
 /******************************************************************************/
@@ -322,91 +244,95 @@ const backgroundImageURLFromElement = function(elem) {
 //   Limit returned string to 1024 characters.
 //   Also, return only URLs which will be seen by an HTTP observer.
 
-const resourceURLFromElement = function(elem) {
+const resourceURLsFromElement = function(elem) {
+    const urls = [];
     const tagName = elem.localName;
     const prop = netFilter1stSources[tagName];
-    if ( prop ) {
-        let src = '';
-        {
-            let s = elem[prop];
-            if ( typeof s === 'string' && /^https?:\/\//.test(s) ) {
-                src = s.slice(0, 1024);
-            }
+    if ( prop === undefined ) {
+        const url = backgroundImageURLFromElement(elem);
+        if ( url !== '' ) { urls.push(url); }
+        return urls;
+    }
+    {
+        const s = elem[prop];
+        if ( typeof s === 'string' && /^https?:\/\//.test(s) ) {
+            urls.push(trimFragmentFromURL(s.slice(0, 1024)));
         }
-        if ( typeof elem.srcset === 'string' && elem.srcset !== '' ) {
-            const ss = [];
-            for ( let s of elem.srcset.split(',') ) {
-                s = s.trim();
-                const pos = s.indexOf(' ');
-                if ( pos !== -1 ) { s = s.slice(0, pos); }
-                const parsedURL = new URL(s, document.baseURI);
-                if ( parsedURL.pathname.length > 1 ) {
-                    ss.push(parsedURL.href);
-                }
-            }
-            if ( ss.length !== 0 ) {
-                if ( src !== '' ) {
-                    ss.push(src);
-                }
-                src = mergeStrings(ss);
-            }
+    }
+    resourceURLsFromSrcset(elem, urls);
+    return urls;
+};
+
+// https://html.spec.whatwg.org/multipage/images.html#parsing-a-srcset-attribute
+// https://github.com/uBlockOrigin/uBlock-issues/issues/1071
+const resourceURLsFromSrcset = function(elem, out) {
+    let srcset = elem.srcset;
+    if ( typeof srcset !== 'string' || srcset === '' ) { return; }
+    for(;;) {
+        // trim whitespace
+        srcset = srcset.trim();
+        if ( srcset.length === 0 ) { break; }
+        // abort in case of leading comma
+        if ( /^,/.test(srcset) ) { break; }
+        // collect and consume all non-whitespace characters
+        let match = /^\S+/.exec(srcset);
+        if ( match === null ) { break; }
+        srcset = srcset.slice(match.index + match[0].length);
+        let url = match[0];
+        // consume descriptor, if any
+        if ( /,$/.test(url) ) {
+            url = url.replace(/,$/, '');
+            if ( /,$/.test(url) ) { break; }
+        } else {
+            match = /^[^,]*(?:\(.+?\))?[^,]*(?:,|$)/.exec(srcset);
+            if ( match === null ) { break; }
+            srcset = srcset.slice(match.index + match[0].length);
         }
-        return src;
+        const parsedURL = new URL(url, document.baseURI);
+        if ( parsedURL.pathname.length === 0 ) { continue; }
+        out.push(trimFragmentFromURL(parsedURL.href));
     }
-    return backgroundImageURLFromElement(elem);
 };
 
 /******************************************************************************/
 
-const netFilterFromUnion = function(toMergeURL, out) {
-    const parsedURL = new URL(toMergeURL, document.baseURI);
-
-    toMergeURL = parsedURL.pathname + parsedURL.search;
-
+const netFilterFromUnion = function(patternIn, out) {
     // Reset reference filter when dealing with unrelated URLs
+    const currentHostname = self.location.hostname;
     if (
         lastNetFilterUnion === '' ||
-        parsedURL.host === '' ||
-        parsedURL.host !== lastNetFilterHostname
+        currentHostname === '' ||
+        currentHostname !== lastNetFilterHostname
     ) {
-        lastNetFilterHostname = parsedURL.host;
-        lastNetFilterUnion = toMergeURL;
-        vAPI.messaging.send(
-            'elementPicker',
-            {
-                what: 'elementPickerEprom',
-                lastNetFilterSession: lastNetFilterSession,
-                lastNetFilterHostname: lastNetFilterHostname,
-                lastNetFilterUnion: lastNetFilterUnion
-            }
-        );
+        lastNetFilterHostname = currentHostname;
+        lastNetFilterUnion = patternIn;
+        vAPI.messaging.send('elementPicker', {
+            what: 'elementPickerEprom',
+            lastNetFilterSession,
+            lastNetFilterHostname,
+            lastNetFilterUnion,
+        });
         return;
     }
 
     // Related URLs
-    lastNetFilterHostname = parsedURL.host;
-
-    let mergedURL = mergeStrings([ toMergeURL, lastNetFilterUnion ]);
-    if ( mergedURL !== '/*' && mergedURL !== toMergeURL ) {
-        const filter = '||' + lastNetFilterHostname + mergedURL;
+    lastNetFilterHostname = currentHostname;
+    let patternOut = mergeStrings([ patternIn, lastNetFilterUnion ]);
+    if ( patternOut !== '/*' && patternOut !== patternIn ) {
+        const filter = `||${patternOut}`;
         if ( out.indexOf(filter) === -1 ) {
             out.push(filter);
         }
-    } else {
-        mergedURL = toMergeURL;
+        lastNetFilterUnion = patternOut;
     }
-    lastNetFilterUnion = mergedURL;
 
     // Remember across element picker sessions
-    vAPI.messaging.send(
-        'elementPicker',
-        {
-            what: 'elementPickerEprom',
-            lastNetFilterSession: lastNetFilterSession,
-            lastNetFilterHostname: lastNetFilterHostname,
-            lastNetFilterUnion: lastNetFilterUnion
-        }
-    );
+    vAPI.messaging.send('elementPicker', {
+        what: 'elementPickerEprom',
+        lastNetFilterSession,
+        lastNetFilterHostname,
+        lastNetFilterUnion,
+    });
 };
 
 /******************************************************************************/
@@ -416,8 +342,8 @@ const netFilterFromUnion = function(toMergeURL, out) {
 const netFilterFromElement = function(elem) {
     if ( elem === null ) { return 0; }
     if ( elem.nodeType !== 1 ) { return 0; }
-    let src = resourceURLFromElement(elem);
-    if ( src === '' ) { return 0; }
+    const urls = resourceURLsFromElement(elem);
+    if ( urls.length === 0 ) { return 0; }
 
     if ( candidateElements.indexOf(elem) === -1 ) {
         candidateElements.push(elem);
@@ -426,13 +352,11 @@ const netFilterFromElement = function(elem) {
     const candidates = netFilterCandidates;
     const len = candidates.length;
 
-    // Remove fragment
-    let pos = src.indexOf('#');
-    if ( pos !== -1 ) {
-        src = src.slice(0, pos);
+    for ( let i = 0; i < urls.length; i++ ) {
+        urls[i] = urls[i].replace(/^https?:\/\//, '');
     }
+    const pattern = mergeStrings(urls);
 
-    const filter = src.replace(/^https?:\/\//, '||');
 
     if ( bestCandidateFilter === null ) {
         bestCandidateFilter = {
@@ -442,16 +366,16 @@ const netFilterFromElement = function(elem) {
         };
     }
 
-    candidates.push(filter);
+    candidates.push(`||${pattern}`);
 
     // Suggest a less narrow filter if possible
-    pos = filter.indexOf('?');
+    const pos = pattern.indexOf('?');
     if ( pos !== -1 ) {
-        candidates.push(filter.slice(0, pos));
+        candidates.push(`||${pattern.slice(0, pos)}`);
     }
 
     // Suggest a filter which is a result of combining more than one URL.
-    netFilterFromUnion(src, candidates);
+    netFilterFromUnion(pattern, candidates);
 
     return candidates.length - len;
 };
@@ -744,7 +668,7 @@ const filterToDOMInterface = (( ) => {
         }
         let reFilter = null;
         try {
-            reFilter = new RegExp(reStr);
+            reFilter = new RegExp(reStr, 'i');
         }
         catch (e) {
             return out;
@@ -755,18 +679,14 @@ const filterToDOMInterface = (( ) => {
             Object.keys(netFilter1stSources).join()
         );
         for ( const elem of elems ) {
-            let srcProp = netFilter1stSources[elem.localName];
-            let src = elem[srcProp];
-            if ( typeof src !== 'string' || src.length === 0 ) {
-                if (
-                    typeof elem.srcset === 'string' &&
-                    elem.srcset !== '' &&
-                    typeof elem.currentSrc === 'string'
-                ) {
-                    src = elem.currentSrc;
-                }
-            }
-            if ( src && reFilter.test(src) ) {
+            const srcProp = netFilter1stSources[elem.localName];
+            const src = elem[srcProp];
+            if (
+                typeof src === 'string' &&
+                    reFilter.test(src) ||
+                typeof elem.currentSrc === 'string' &&
+                    reFilter.test(elem.currentSrc)
+            ) {
                 out.push({
                     type: 'network',
                     elem: elem,
@@ -828,11 +748,11 @@ const filterToDOMInterface = (( ) => {
         let elems;
         try {
             const o = JSON.parse(raw);
-            if ( o.style ) {
+            if ( o.action === 'style' ) {
                 elems = document.querySelectorAll(
-                    o.style[0].replace(rePseudoElements, '')
+                    o.selector.replace(rePseudoElements, '')
                 );
-                lastAction = o.style[0] + ' {' + o.style[1] + '}';
+                lastAction = o.selector + ' {' + o.tasks[0][1] + '}';
             } else if ( o.tasks ) {
                 elems = vAPI.domFilterer.createProceduralFilter(o).exec();
             }
@@ -854,7 +774,7 @@ const filterToDOMInterface = (( ) => {
         applied = false,
         previewing = false;
 
-    const queryAll = function(filter, callback) {
+    const queryAll = async function(filter, callback) {
         filter = filter.trim();
         if ( filter === lastFilter ) {
             callback(lastResultset);
@@ -875,23 +795,20 @@ const filterToDOMInterface = (( ) => {
             callback(lastResultset);
             return;
         }
-        const selector = filter.slice(2);
-        lastResultset = fromPlainCosmeticFilter(selector);
+        lastResultset = fromPlainCosmeticFilter(filter.slice(2));
         if ( lastResultset ) {
             if ( previewing ) { apply(); }
             callback(lastResultset);
             return;
         }
         // Procedural cosmetic filter
-        vAPI.messaging.send(
-            'elementPicker',
-            { what: 'compileCosmeticFilterSelector', selector: selector },
-            response => {
-                lastResultset = fromCompiledCosmeticFilter(response);
-                if ( previewing ) { apply(); }
-                callback(lastResultset);
-            }
-        );
+        const response = await vAPI.messaging.send('elementPicker', {
+            what: 'compileCosmeticFilterSelector',
+            selector: filter,
+        });
+        lastResultset = fromCompiledCosmeticFilter(response);
+        if ( previewing ) { apply(); }
+        callback(lastResultset);
     };
 
     // https://github.com/gorhill/uBlock/issues/1629
@@ -1190,17 +1107,14 @@ const onDialogClicked = function(ev) {
         filterToDOMInterface.preview(false);
         userFilterFromCandidate((filter = undefined, isCosmetic = false) => {
             if ( filter === undefined ) { return; }
-            vAPI.messaging.send(
-                'elementPicker',
-                {
-                    what: 'createUserFilter',
-                    autoComment: true,
-                    filters: filter,
-                    origin: window.location.origin,
-                    pageDomain: window.location.hostname,
-                    killCache: isCosmetic === false,
-                }
-            );
+            vAPI.messaging.send('elementPicker', {
+                what: 'createUserFilter',
+                autoComment: true,
+                filters: filter,
+                origin: window.location.origin,
+                pageDomain: window.location.hostname,
+                killCache: isCosmetic === false,
+            });
             filterToDOMInterface.preview(rawFilterFromTextarea(), true);
             stopPicker();
         });
@@ -1500,7 +1414,10 @@ const svgListening = function(on) {
 
 const onKeyPressed = function(ev) {
     // Delete
-    if ( ev.key === 'Delete' && pickerBody.classList.contains('zap') ) {
+    if (
+        (ev.key === 'Delete' || ev.key === 'Backspace') &&
+        pickerBody.classList.contains('zap')
+    ) {
         ev.stopPropagation();
         ev.preventDefault();
         zap();
@@ -1528,6 +1445,65 @@ const onScrolled = function() {
 
 /******************************************************************************/
 
+const onStartMoving = (( ) => {
+    let mx0 = 0, my0 = 0;
+    let mx1 = 0, my1 = 0;
+    let r0 = 0, b0 = 0;
+    let rMax = 0, bMax = 0;
+    let timer;
+
+    const move = ( ) => {
+        timer = undefined;
+        let r1 = Math.min(Math.max(r0 - mx1 + mx0, 4), rMax);
+        let b1 = Math.min(Math.max(b0 - my1 + my0, 4), bMax);
+        dialog.style.setProperty('right', `${r1}px`, 'important');
+        dialog.style.setProperty('bottom', `${b1}px`, 'important');
+    };
+
+    const moveAsync = ev => {
+        if ( ev.isTrusted === false ) { return; }
+        ev.preventDefault();
+        ev.stopPropagation();
+        if ( timer !== undefined ) { return; }
+        mx1 = ev.pageX;
+        my1 = ev.pageY;
+        timer = self.requestAnimationFrame(move);
+    };
+
+    const stop = ev => {
+        if ( ev.isTrusted === false ) { return; }
+        if ( dialog.classList.contains('moving') === false ) { return; }
+        dialog.classList.remove('moving');
+        const pickerWin = pickerRoot.contentWindow;
+        pickerWin.removeEventListener('mousemove', moveAsync, { capture: true });
+        pickerWin.removeEventListener('mouseup', stop, { capture: true, once: true });
+        ev.preventDefault();
+        ev.stopPropagation();
+    };
+
+    return function(ev) {
+        if ( ev.isTrusted === false ) { return; }
+        const target = dialog.querySelector('#toolbar');
+        if ( ev.target !== target ) { return; }
+        if ( dialog.classList.contains('moving') ) { return; }
+        mx0 = ev.pageX; my0 = ev.pageY;
+        const pickerWin = pickerRoot.contentWindow;
+        const style = pickerWin.getComputedStyle(dialog);
+        r0 = parseInt(style.right, 10);
+        b0 = parseInt(style.bottom, 10);
+        const rect = dialog.getBoundingClientRect();
+        rMax = pickerBody.clientWidth - 4 - rect.width ;
+        bMax = pickerBody.clientHeight - 4 - rect.height;
+        dialog.classList.add('moving');
+        pickerWin.addEventListener('mousemove', moveAsync, { capture: true });
+        pickerWin.addEventListener('mouseup', stop, { capture: true, once: true });
+        ev.preventDefault();
+        ev.stopPropagation();
+    };
+})();
+
+/******************************************************************************/
+
 const pausePicker = function() {
     pickerBody.classList.add('paused');
     svgListening(false);
@@ -1564,20 +1540,11 @@ const stopPicker = function() {
     vAPI.domFilterer.unexcludeNode(pickerRoot);
 
     window.removeEventListener('scroll', onScrolled, true);
-    pickerRoot.contentWindow.removeEventListener('keydown', onKeyPressed, true);
-    taCandidate.removeEventListener('input', onCandidateChanged);
-    dialog.removeEventListener('click', onDialogClicked);
     svgListening(false);
-    svgRoot.removeEventListener('click', onSvgClicked);
-    svgRoot.removeEventListener('touchstart', onSvgTouchStartStop);
-    svgRoot.removeEventListener('touchend', onSvgTouchStartStop);
     pickerRoot.parentNode.removeChild(pickerRoot);
-    pickerRoot.removeEventListener('load', stopPicker);
-    pickerRoot =
-    pickerBody =
-    dialog =
-    svgRoot = svgOcean = svgIslands =
-    taCandidate = null;
+    pickerRoot = pickerBody =
+        svgRoot = svgOcean = svgIslands =
+        dialog = taCandidate = null;
 
     window.focus();
 };
@@ -1623,6 +1590,8 @@ const startPicker = function(details) {
     taCandidate = dialog.querySelector('textarea');
     taCandidate.addEventListener('input', onCandidateChanged);
 
+    dialog.querySelector('#toolbar').addEventListener('mousedown', onStartMoving);
+
     svgRoot = pickerBody.querySelector('svg');
     svgOcean = svgRoot.firstChild;
     svgIslands = svgRoot.lastChild;
@@ -1647,8 +1616,12 @@ const startPicker = function(details) {
     highlightElements([], true);
 
     // Try using mouse position
-    if ( details.clientX !== -1 ) {
-        if ( filtersFrom(details.clientX, details.clientY) !== 0 ) {
+    if (
+        details.mouse &&
+        typeof vAPI.mouseClick.x === 'number' &&
+        vAPI.mouseClick.x > 0
+    ) {
+        if ( filtersFrom(vAPI.mouseClick.x, vAPI.mouseClick.y) !== 0 ) {
             showDialog();
             return;
         }
@@ -1697,14 +1670,12 @@ const startPicker = function(details) {
 
 /******************************************************************************/
 
-const bootstrapPicker = function() {
-    pickerRoot.removeEventListener('load', bootstrapPicker);
+const bootstrapPicker = async function() {
     vAPI.shutdown.add(stopPicker);
-    vAPI.messaging.send(
-        'elementPicker',
-        { what: 'elementPickerArguments' },
-        startPicker
-    );
+    const details = await vAPI.messaging.send('elementPicker', {
+        what: 'elementPickerArguments',
+    });
+    startPicker(details);
 };
 
 /******************************************************************************/
@@ -1723,6 +1694,8 @@ const pickerCSSStyle = [
     'margin: 0',
     'max-height: none',
     'max-width: none',
+    'min-height: unset',
+    'min-width: unset',
     'opacity: 1',
     'outline: 0',
     'padding: 0',
@@ -1762,7 +1735,11 @@ vAPI.userStylesheet.apply();
 // https://github.com/gorhill/uBlock/issues/2060
 vAPI.domFilterer.excludeNode(pickerRoot);
 
-pickerRoot.addEventListener('load', bootstrapPicker);
+pickerRoot.addEventListener(
+    'load',
+    ( ) => { bootstrapPicker(); },
+    { once: true }
+);
 document.documentElement.appendChild(pickerRoot);
 
 /******************************************************************************/
