@@ -331,7 +331,7 @@ const matchBucket = function(url, hostname, bucket, start) {
     switch ( name ) {
     case 'advancedUserEnabled':
         if ( value === true ) {
-            us.dynamicFilteringEnabled = true;
+            us.popupPanelSections |= 0b11111;
         }
         break;
     case 'autoUpdate':
@@ -414,35 +414,28 @@ const matchBucket = function(url, hostname, bucket, start) {
 
 /******************************************************************************/
 
-µBlock.elementPickerExec = function(tabId, targetElement, zap) {
+µBlock.elementPickerExec = async function(tabId, targetElement, zap = false) {
     if ( vAPI.isBehindTheSceneTabId(tabId) ) { return; }
 
-    this.epickerTarget = targetElement || '';
-    this.epickerZap = zap || false;
+    this.epickerArgs.target = targetElement || '';
+    this.epickerArgs.zap = zap;
 
     // https://github.com/uBlockOrigin/uBlock-issues/issues/40
     //   The element picker needs this library
-    vAPI.tabs.injectScript(
-        tabId,
-        {
-            file: '/lib/diff/swatinem_diff.js',
-            runAt: 'document_end'
-        }
-    );
+    vAPI.tabs.executeScript(tabId, {
+        file: '/lib/diff/swatinem_diff.js',
+        runAt: 'document_end',
+    });
+
+    await vAPI.tabs.executeScript(tabId, {
+        file: '/js/scriptlets/element-picker.js',
+        runAt: 'document_end',
+    });
 
     // https://github.com/uBlockOrigin/uBlock-issues/issues/168
     //   Force activate the target tab once the element picker has been
     //   injected.
-    vAPI.tabs.injectScript(
-        tabId,
-        {
-            file: '/js/scriptlets/element-picker.js',
-            runAt: 'document_end'
-        },
-        ( ) => {
-            vAPI.tabs.select(tabId);
-        }
-    );
+    vAPI.tabs.select(tabId);
 };
 
 /******************************************************************************/
@@ -638,10 +631,11 @@ const matchBucket = function(url, hostname, bucket, start) {
 //   cosmetic filters.
 
 µBlock.logCosmeticFilters = function(tabId, frameId) {
-    vAPI.tabs.injectScript(tabId, {
+    vAPI.tabs.executeScript(tabId, {
         file: '/js/scriptlets/cosmetic-logger.js',
         frameId: frameId,
-        runAt: 'document_start'
+        matchAboutBlank: true,
+        runAt: 'document_start',
     });
 };
 
@@ -694,15 +688,15 @@ const matchBucket = function(url, hostname, bucket, start) {
             }
             pendingEntries.set(key, new Entry(tabId, scriptlet, callback));
         }
-        vAPI.tabs.injectScript(tabId, {
-            file: '/js/scriptlets/' + scriptlet + '.js'
+        vAPI.tabs.executeScript(tabId, {
+            file: `/js/scriptlets/${scriptlet}.js`
         });
     };
 
     // TODO: think about a callback mechanism.
     const injectDeep = function(tabId, scriptlet) {
-        vAPI.tabs.injectScript(tabId, {
-            file: '/js/scriptlets/' + scriptlet + '.js',
+        vAPI.tabs.executeScript(tabId, {
+            file: `/js/scriptlets/${scriptlet}.js`,
             allFrames: true
         });
     };
