@@ -6,7 +6,7 @@ echo "*** uBlock: Importing from Crowdin archive"
 
 SRC=~/Downloads/crowdin
 rm -r $SRC
-unzip -q ~/Downloads/ublock.zip -d $SRC
+unzip -q ~/Downloads/uBlock.zip -d $SRC
 
 DES=./src/_locales
 cp $SRC/ar/messages.json    $DES/ar/messages.json
@@ -35,6 +35,7 @@ cp $SRC/he/messages.json    $DES/he/messages.json
 cp $SRC/hi/messages.json    $DES/hi/messages.json
 cp $SRC/hr/messages.json    $DES/hr/messages.json
 cp $SRC/hu/messages.json    $DES/hu/messages.json
+cp $SRC/hy-AM/messages.json $DES/hy/messages.json
 cp $SRC/id/messages.json    $DES/id/messages.json
 cp $SRC/it/messages.json    $DES/it/messages.json
 cp $SRC/ja/messages.json    $DES/ja/messages.json
@@ -49,6 +50,7 @@ cp $SRC/mr/messages.json    $DES/mr/messages.json
 cp $SRC/ms/messages.json    $DES/ms/messages.json
 cp $SRC/nb/messages.json    $DES/nb/messages.json
 cp $SRC/nl/messages.json    $DES/nl/messages.json
+cp $SRC/oc/messages.json    $DES/oc/messages.json
 cp $SRC/pl/messages.json    $DES/pl/messages.json
 cp $SRC/pt-BR/messages.json $DES/pt_BR/messages.json
 cp $SRC/pt-PT/messages.json $DES/pt_PT/messages.json
@@ -104,6 +106,7 @@ cp $SRC/he/description.txt    $DES/description-he.txt
 cp $SRC/hi/description.txt    $DES/description-hi.txt
 cp $SRC/hr/description.txt    $DES/description-hr.txt
 cp $SRC/hu/description.txt    $DES/description-hu.txt
+cp $SRC/hy-AM/description.txt $DES/description-hy.txt
 cp $SRC/id/description.txt    $DES/description-id.txt
 cp $SRC/it/description.txt    $DES/description-it.txt
 cp $SRC/ja/description.txt    $DES/description-ja.txt
@@ -118,6 +121,7 @@ cp $SRC/ms/description.txt    $DES/description-ms.txt
 cp $SRC/mr/description.txt    $DES/description-mr.txt
 cp $SRC/nb/description.txt    $DES/description-nb.txt
 cp $SRC/nl/description.txt    $DES/description-nl.txt
+cp $SRC/oc/description.txt    $DES/description-oc.txt
 cp $SRC/pl/description.txt    $DES/description-pl.txt
 cp $SRC/pt-BR/description.txt $DES/description-pt_BR.txt
 cp $SRC/pt-PT/description.txt $DES/description-pt_PT.txt
